class WoWWDB
  def initialize(filename)
    @file = open(filename,"rb")
    header = @file.read(20).unpack("a4ia4ii")
    @header = {
      :sig => header[0].reverse,
      :version => header[1],
      :language => header[2].reverse
    }
    @data = {}
    readData
  end
  
  def entries
    @entries ||= @data.keys
  end
  
  def entry(uid)
    e = @data[uid]
    @file.seek(e[:offset],IO::SEEK_SET)
    @file.read(e[:size])
  end
  
  private
  def readData
    uid = -1
    size = -1
    while (uid != 0 && size != 0) do
      tmp = @file.read(8).unpack("ii")
      uid = tmp[0]
      size = tmp[1]
      offset = @file.tell
      @file.seek(size,IO::SEEK_CUR)
      if (uid != 0 && size != 0)
        @data[uid] = { :size => size, :offset => offset }
      end
    end
  end
end

class GarbageFu
  def main(wowDir)
    
    @added = 0
    @removed = 0
    @changed = 0
    
    @wowDir = wowDir
    gfItems = parsePrices
    @cache = WoWWDB.new(@wowDir + "/Cache/WDB/enUS/itemcache.wdb")
    dbItems = parseCache
    items = mergeItems(gfItems,dbItems)
    writePrices(items)
    
    print "Old Prices:    " + (gfItems.length.to_s) + "\n"
    print "WDB Prices:    " + (dbItems.length.to_s) + "\n"
    print "Merged Prices: " + (items.length.to_s) + "\n"
    
    print "\n"
    print "Added:         " + (@added.to_s) + "\n"
    print "Removed:       " + (@removed.to_s) + "\n"
    print "Changed:       " + (@changed.to_s) + "\n"
  end
  
  def mergeItems(a,b)
    out = {}
    a.each { |k,v| out[k]=v if (v != 0) }
    b.each do |k,v|
      if v == 0
        if (out[k] != nil)
          out.delete(k)
          @removed = @removed + 1
        end
      else
        if (out[k] == nil)
          @added = @added + 1
        elsif (out[k] != v)
          @changed = @changed + 1
        end
        out[k]=v
      end
    end
    out
  end
  
  def parseCache
    out = {}
    @cache.entries.each do |itemid|
      data = @cache.entry(itemid).unpack("i2x4Z*Z*Z*Z*i2x4i2")
      if (data[7] < 2)
        out[itemid] = data[9] || 0
      end
    end
    out
  end
  
  def writePrices(items)
    f = open(@wowDir + "/Interface/AddOns/FuBar_GarbageFu_Prices/GarbageFu_Prices.lua","w")
    data = items.collect { |id,price| (id.to_s) + ":" + (price.to_s) }
    f.write("GarbageFu_PriceDb = \" " + (data.join(" ")) + " \"\n" )
    f.close
    
    f = open(@wowDir + "/Interface/AddOns/FuBar_GarbageFu_Prices/GarbageFu_Prices_Table.lua","w")
    f.write("GarbageFu_PriceTbl={\n")
    items.keys.sort.each do |id|
      f.write(sprintf("[%i]=%i,\n",id,items[id]))
    end
    f.write("}\n")
    f.close
  end
  
  def parsePrices
    f = open(@wowDir + "/Interface/AddOns/FuBar_GarbageFu_Prices/GarbageFu_Prices.lua","r")
    line = f.readline
    f.close
    line = line[21..-3]
    parts = line.split(" ")
    items = {}
    parts.each do |n|
      v = n.split(":")
      items[v[0].to_i]=(v[1].to_i)
    end
    items
  end
end

gfu = GarbageFu.new
gfu.main(ARGV[0] || "../../..")